﻿using System;
using System.Collections.Generic;
using System.Linq;
using ff14bot;
using ff14bot.Helpers;
using TreeSharp;
using Action = TreeSharp.Action;

namespace Zekken.Avoidance
{
    internal class AvoidanceManager
    {
        private SpellLogger logger = new SpellLogger();
        private AvoidanceMover mover = new AvoidanceMover();
        private SpellCastDetector castDetector;
        private HashSet<uint> ignoredSpells = new HashSet<uint>();
        private List<SpellCast> castsBeingAvoided = new List<SpellCast>();
        private SpellShapeDatabase database;

        public AvoidanceManager(SpellShapeDatabase database)
        {
            this.database = database;
            castDetector = new SpellCastDetector(database);
        }

        public Composite GetAvoidanceBehavior()
        {
            return new Action(r => AvoidTelegraph());
        }

        private RunStatus AvoidTelegraph()
        {
            var newCasts = castDetector.GetNewCasts();
            var presentCasts = castDetector.spellCasts;
            castsBeingAvoided.RemoveAll(c => !presentCasts.Contains(c));

            ProcessUnknownCasts(newCasts);
            newCasts = GetAvoidable(newCasts);

            if (newCasts.Count == 0)
            {
                return mover.Continue(castsBeingAvoided);
            }

            var activeCastPairs = newCasts
                .Select(c => new Tuple<SpellCast, AvoidanceVector>(c, VectorSelector.GetAvoidanceVector(c)))
                .Where(p => p.Item2.IsActive)
                .ToList();

            var newCastsBeingAvoided = castDetector.spellCasts
                .Where(c => activeCastPairs.Any(p => p.Item1 == c))
                .ToList();

            castsBeingAvoided.AddRange(newCastsBeingAvoided);
            mover.Start(activeCastPairs.Select(p => p.Item2));

            return RunStatus.Success;
        }

        private void ProcessUnknownCasts(IEnumerable<SpellCast> casts)
        {
            var unknownCast = casts
                .FirstOrDefault(sc =>  sc.Caster.CurrentTargetId == Core.Me.ObjectId
                    && sc.Shape == SpellShape.None 
                    && !ignoredSpells.Contains(sc.SpellId));

            if (unknownCast == null) { return; }

            logger.LogSpell(unknownCast.SpellId, unknownCast.Name);
            Logging.Write(Common.LogColor, string.Format("[Zekken] Unknown cast logged: {0}.",
                unknownCast.Name));

            ignoredSpells.Add(unknownCast.SpellId);
        }

        private List<SpellCast> GetAvoidable(IEnumerable<SpellCast> casts)
        {
            var avoidable = casts
                .Where(sc => database.Contains(sc.SpellId))
                .Where(sc => !sc.Shape.HasFlag(SpellShape.NotTelegraphed))
                .ToList();

            return avoidable;
        }
    }
}
