﻿using System;
using ff14bot.Behavior;
using ff14bot.Helpers;
using ff14bot.Interfaces;
using TreeSharp;

namespace Zekken.Avoidance
{
    public class AvoidancePlugin : IBotPlugin
    {
        private AvoidanceManager spellAvoidance;
        private Composite avoidanceBehavoir;
        private SpellShapeDatabase database;
        private bool enabled;

        #region Plugin Implementation
        public string Author
        {
            get { return " Saga"; }
        }

        public string ButtonText
        {
            get { return "Compile"; }
        }

        public string Description
        {
            get { return "Telegraphed spell avoidance."; }
        }

        public string Name
        {
            get { return " Zekken"; }
        }

        public void OnButtonPress()
        {
            if (enabled) { UnhookAvoidance(); }
            
            SpellShapeCompiler.CompileShapes();
            database = new SpellShapeDatabase();
            database.Load();

            if (enabled) { HookAvoidance(); }
        }

        public void OnDisabled()
        {
            TreeHooks.Instance.OnHooksCleared -= OnHooksCleared;

            if (enabled) { UnhookAvoidance(); }
            enabled = false;
        }

        public void OnEnabled()
        {
            TreeHooks.Instance.OnHooksCleared += OnHooksCleared;

            if (!enabled) { HookAvoidance(); }
            enabled = true;
        }

        public void OnInitialize()
        {
            database = new SpellShapeDatabase();
            database.Load();
        }

        public void OnPulse()
        {
            
        }

        public void OnShutdown()
        {
            if (enabled) { UnhookAvoidance(); }
            enabled = false;
        }

        public Version Version
        {
            get { return new Version(1, 3); }
        }

        public bool WantButton
        {
            get { return true; }
        }

        public bool Equals(IBotPlugin other)
        {
            return other.Name == Name && other.Author == Author;
        }
        #endregion

        private void OnHooksCleared(object sender, EventArgs args)
        {
            HookAvoidance();
        }

        private void HookAvoidance()
        {
            spellAvoidance = new AvoidanceManager(database);
            avoidanceBehavoir = spellAvoidance.GetAvoidanceBehavior();

            TreeHooks.Instance.AddHook("PreCombatLogic", avoidanceBehavoir);
            Logging.Write(Common.LogColor, "[Zekken] Avoidance hooked.");
        }

        private void UnhookAvoidance()
        {
            spellAvoidance = null;

            if (avoidanceBehavoir != null) { TreeHooks.Instance.RemoveHook("PreCombatLogic", avoidanceBehavoir); }
            avoidanceBehavoir = null;
            
            Logging.Write(Common.LogColor, "[Zekken] Avoidance unhooked.");
        }
    }
}
