﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;
using System.Threading.Tasks;
using ff14bot;
using ff14bot.Helpers;
using ff14bot.Objects;

namespace Zekken.Avoidance
{
    public class SpellLogger
    {
        private object locker = new object();
        private const uint WM_KEYDOWN = 0x100;
        private const uint WM_KEYUP = 0x0101;
        private IntPtr windowHandle;
        private volatile bool enabled = true;
        private volatile bool uiHidden;
        private const int WAIT_BEFORE_SCREENSHOT = 250;

        public SpellLogger()
        {
            const string directory = Common.DEFAULT_SCREENSHOT_PATH;
            windowHandle = Core.Memory.Process.MainWindowHandle;

            if (Directory.Exists(directory)) { return; }

            try { Directory.CreateDirectory(directory); }
            catch
            {
                Logging.Write(Common.ErrorColor, "[Zekken] Could not create screenshot directory.");
                enabled = false;
            }
        }

        public void LogSpell(uint spellId, string spellname)
        {
            if (!enabled) { return; }

            var name = string.Format("{0} - {1}", spellname, spellId);
            Logging.Write(Common.LogColor, string.Format("[Zekken] Taking screenshot of unknown spell {0}.", name));

            Task.Run(() => TakePicture(name));
        }

        private void TakePicture(string name)
        {
            Thread.Sleep(WAIT_BEFORE_SCREENSHOT);

            Bitmap image;

            HideOrShowUi(true);

            Thread.Sleep(200);

            try { image = PrintWindow(windowHandle); }
            catch (Exception e)
            {
                Logging.Write(Common.ErrorColor, string.Format("[Zekken] Could not save screenshot because: {0}", e));
                enabled = false;
                return;
            }

            HideOrShowUi(false);

            var path = Path.Combine(Common.DEFAULT_SCREENSHOT_PATH, name) + ".png";
            try
            {
                using (var file = new FileStream(path, FileMode.Create)) { image.Save(file, ImageFormat.Png); }
            }
            catch (Exception e)
            {
                Logging.Write(Common.ErrorColor, string.Format("[Zekken] Could not save screenshot because: {0}", e));
                enabled = false;
                return;
            }

            Logging.Write(Common.LogColor, string.Format("[Zekken] Screenshot captured for {0}.", name));
        }

        private void HideOrShowUi(bool hide)
        {
            lock (locker)
            {
                if (uiHidden && hide) { return; }

                PostMessage(windowHandle, WM_KEYDOWN, (IntPtr)(Keys.Scroll), IntPtr.Zero);
                PostMessage(windowHandle, WM_KEYUP, (IntPtr)(Keys.Scroll), IntPtr.Zero);

                uiHidden = hide;
            }
        }

        [DllImport("user32.dll")]
        private static extern bool GetWindowRect(IntPtr hWnd, out Rect lpRect);
        [DllImport("user32.dll")]
        private static extern bool PrintWindow(IntPtr hWnd, IntPtr hdcBlt, int nFlags);
        [DllImport("user32.dll")]
        private static extern IntPtr PostMessage(IntPtr hWnd, uint msg, IntPtr wParam, IntPtr lParam);

        private static Bitmap PrintWindow(IntPtr hwnd)
        {
            Rect rc;
            GetWindowRect(hwnd, out rc);

            Bitmap bmp = new Bitmap(rc.Width, rc.Height, PixelFormat.Format16bppRgb555);
            Graphics gfxBmp = Graphics.FromImage(bmp);
            IntPtr hdcBitmap = gfxBmp.GetHdc();

            PrintWindow(hwnd, hdcBitmap, 0);

            gfxBmp.ReleaseHdc(hdcBitmap);
            gfxBmp.Dispose();

            return bmp;
        }
    }
}
