﻿using System;
using Clio.Utilities;
using ff14bot;
using ff14bot.Helpers;

namespace Zekken.Avoidance
{
    internal static class VectorSelector
    {
        private const float LARGE_MAGNITUDE = 25f;
        private const float MEDIUM_MAGNITUDE = 15f;
        private const float SMALL_MAGNITUDE = 7f;

        private const SpellShape DIRECTIONAL_FROM_CASTER =
            SpellShape.DirForward | SpellShape.TargetedFromCaster;

        private const SpellShape CIRCLE_ON_CASTER = SpellShape.Circle | SpellShape.TargetedFromCaster;

        public static AvoidanceVector GetAvoidanceVector(SpellCast cast)
        {
            return GetAvoidanceVector(cast.SpellId, cast.Shape, cast.CasterLocation,
                cast.CasterHeading, cast.TargetLocation);
        }

        public static AvoidanceVector GetAvoidanceVector(uint spellID, SpellShape shape, Vector3 enemyLocation, float enemyHeading, Vector3 targetLocation)
        {
            Logging.Write(Common.LogColor, string.Format("[Zekken] Avoiding spell {1} with shape {0}.", spellID, shape));

            var myLocation = Core.Me.Location;
            var originPoint = shape.HasFlag(SpellShape.TargetedOnTarget) ? targetLocation : enemyLocation;
            var reach = GetReach(shape, myLocation, originPoint);

            if (reach <= 0f) { return AvoidanceVector.Zero; }

            if (shape.HasFlag(CIRCLE_ON_CASTER))
            {
                return VectorTools.GetOppositeVector(myLocation,
                    originPoint, reach); 
            }

            if (shape.HasFlag(DIRECTIONAL_FROM_CASTER))
            {
                if (!VectorTools.IsFacing(originPoint, enemyHeading, myLocation))
                {
                    Logging.Write(Common.LogColor, string.Format("[Zekken] The enemy is not facing me. No need to move."));
                    return AvoidanceVector.Zero;
                }

                var distance = myLocation.Distance(originPoint);
                if (distance < 6)
                {
                    return VectorTools.GetStraightVector(myLocation, originPoint, Math.Max(8f, distance * 2));
                }

                return VectorTools.GetOppositeVector(myLocation, originPoint, reach);
            }

            if (shape.HasFlag(SpellShape.TargetedOnTarget))
            {
                return VectorTools.GetOppositeVector(myLocation, originPoint, reach);
            }

            Logging.Write(Common.LogColor, string.Format("[Zekken] I don't know enough about this spell."));
            return AvoidanceVector.Zero;
        }

        private static float GetReach(SpellShape shape, Vector3 myLocation, Vector3 originPoint)
        {
            var currentDistance = myLocation.Distance(originPoint);

            if (shape.HasFlag(SpellShape.ReachLarge))
            {
                return Math.Max(0, LARGE_MAGNITUDE - currentDistance);
            }

            if (shape.HasFlag(SpellShape.ReachMedium))
            {
                return Math.Max(0, MEDIUM_MAGNITUDE - currentDistance);
            }

            return Math.Max(0, SMALL_MAGNITUDE - currentDistance);
        }
    }
}
